#!/bin/csh -f

# ====================================================
# Script for running XBus eVC demo
# 
# Currently only works for MTI and xsim
# 
#   source demo.sh [mti|xsim]
#   source demo.sh -h
# ====================================================
# default values for langauge and simulator
set sim = xsim
setenv APB `sn_which.sh shr_apb`

# parse arguments
while ($#argv > 0)
   switch ($1)
      case -h :
         echo "Script for running APB eVC demo"
         echo "Usage: source demo.sh  [mti|xsim]"
         echo "       source demo.sh -h"
         exit 0
      case mti :
         set sim = mti
         breaksw
      case xsim :
         set sim = xsim
         set lang = verilog
         breaksw
      default :  
         echo "Usage: source demo.sh [mti|xsim]"
         exit 1
   endsw
   shift
end
echo "Running APB demo with $sim simulator"

setenv package_name `sn_which.sh shr_apb`

#check that the APB eVC is correctly installed
if !(-f ${package_name}/PACKAGE_README.txt) then
   echo "APB eVC package is not correctly installed"
   echo "Possibly, SPECMAN_PATH not set properly"
   exit 1
endif

#remove any old MTI work directory
if ($sim == mti) then
   if (-e work) then
      /bin/rm -rf work
   endif
endif

#Create an MTI ini file if necessary
if ($sim == mti) then
   if (! -e ./modelsim.ini) then 
      /bin/cp ${package_name}/examples/modelsim.ini .
   endif
endif

switch ($sim)
   case xsim :
		\rm -rf  work
		specman -c "write stubs -xvhdl ./examples/spexsim.vhd"
		vhan -work work ${APB}/examples/spexsim.vhd
		vhan -work work ${APB}/examples/tb_apb_xsim.vhd
		sn_compile.sh -sim xsim -sim_flags "-top top -work work -t 1ns"
		specview ./xsim_specman -s -i ${APB}/examples/do.cmd
		specview ./xsim_specman -s -i ${APB}/examples/do.cmd
		specview ./xsim_specman -s -i ${APB}/examples/do.cmd
		specview ./xsim_specman -s -i ${APB}/examples/do.cmd
		specview ./xsim_specman -s -i ${APB}/examples/do.cmd
		specview ./xsim_specman -s -i ${APB}/examples/do.cmd
      endif
   case mti :
      vlib work
      if ($lang == vhdl) then
          #Recompile the VHDL
          vcom ${APB}/examples/specman_qvh.vhd
			 vcom ${APB}/examples/tb_apb.vhd

          #Launch Specview and Modelsim
          specview vsim -keepstdout -do ${APB}/examples/sv.do top &
      endif
      breaksw
   default:
      echo "Unknown simulator : $sim"
      exit 1
endsw



