Title        : Top level env unit for eVC
Project      : APB eVC
File name    : shr_apb_env.e
             :
Created On   : 22/04/2002
             :
Developers   : Patrick Oury, Verisity EURL
             :
Purpose      : 
             :
Description  : 
             :
Assumptions  : none
             :
Limitations  : none
             :
Known Errors : none
             :
Notes        : 

----------------------------------------------------------------------------
Revision History :
----------------------------------------------------------------------------

<'

import shr_apb_agent;
import shr_apb_monitor;
import shr_apb_memory_map;

unit shr_apb_env_u like any_env {

    name : shr_apb_bus_name_t;
    bridge_presence : erm_active_passive_t;
    active_bridge_name	: shr_apb_agent_name_t;
    active_slave_names     : list of shr_apb_slave_id_t;
    passive_slave_names  : list of shr_apb_slave_id_t;
    all_slave_names  : list of shr_apb_slave_id_t;
      keep all_slave_names == {passive_slave_names;active_slave_names};
    next_id : int (bits: 16);
        keep soft next_id == 0;
    log_enable          : bool;
      keep soft log_enable == FALSE;

	 reserved_space : shr_apb_reserved_space_s; // for user memory map
	      keep reserved_space.name == name;
	 amba3 : bool;
	  keep soft amba3 == FALSE;
    -- The names of the bus signals
    smp:		shr_apb_signal_map_u is instance;
	 	  keep smp.amba3 == amba3;
        keep smp.name == name;    
    ssmp: list of shr_apb_slave_signal_map_u is instance;
        keep ssmp.size() == all_slave_names.size();
        keep for each (map) in ssmp {
            map.name == name;
            map.id == active_slave_names[index];
			
	};


    -- user will have to precise ACTIVE or PASSIVE BRIDGE
    bus_bridge :  BRIDGE shr_apb_agent_u is instance;
        keep bus_bridge.env == me;
        keep bus_bridge.presence == bridge_presence;
        keep bus_bridge.name == active_bridge_name;
        keep bus_bridge is a ACTIVE BRIDGE shr_apb_agent_u (a) => a.bfm.env == me;


    -- List of all ACTIVE slave agents
    active_slaves : list of ACTIVE SLAVE shr_apb_agent_u is instance;
        keep soft active_slaves.size() == active_slave_names.size();
        keep for each (slave) in active_slaves {
            slave.id == active_slave_names[index];
            slave.env == me;
            slave.bfm.env == me;
            //slave.bfm.bus_monitor == bus_monitor;
        };

    -- List of all PASSIVE slave agents
    passive_slaves : list of PASSIVE SLAVE shr_apb_agent_u is instance;
        keep soft passive_slaves.size() == passive_slave_names.size();
        keep for each (slave) in passive_slaves {
            slave.id == passive_slave_names[index];
            slave.env == me;
        };

    -- List of all slave agents (both passive and eVC-supplied)
    slaves : list of SLAVE shr_apb_agent_u;
        keep slaves == {passive_slaves; active_slaves};


    memory_map : shr_apb_memory_map_u is instance;
	keep soft memory_map.slaves_id  == slaves.id;
	keep soft memory_map.name == name;    


    -- If this field is TRUE then the eVC supplies a protocol checker
    has_protocol_checker : bool;
        keep soft has_protocol_checker == FALSE;

    -- Instatiate the bus monitor
    bus_monitor : shr_apb_bus_monitor_u is instance;
        keep bus_monitor.env == me;
        keep bus_monitor.has_protocol_checker == has_protocol_checker;
        keep bus_monitor.amba3 == amba3;
		  
    reset_asserted : bool;
        keep soft reset_asserted == TRUE;
    
    -- This event gets emitted each time the reset signal changes state. Note
    -- that, depending on how reset is generated, it is possible that this
    -- event will be emitted at time zero.
    event reset_change is change(smp.sig_presetn$) @sim;
    
    -- This event gets emitted when reset is asserted.
    event reset_start is
        true((not reset_asserted) and (smp.sig_presetn$ == 0)) @reset_change;
    
    -- This event gets emitted when reset is de-asserted.
    event reset_end is 
        true(reset_asserted and (smp.sig_presetn$ == 1)) @reset_change;
    
    -- This code ensures that reset_asserted tracks the reset signal.
    on reset_start { reset_asserted = TRUE; };
    on reset_end   { reset_asserted = FALSE; };

    -- This event is the rising edge of the bus clock, unqualified by reset.
    event unqualified_clock_rise is rise(smp.sig_pclk$) @sim;

    -- This event is the falling edge of the bus clock, unqualified by reset.
    event unqualified_clock_fall is fall(smp.sig_pclk$) @sim;
    
    -- This event is the rising edge of the bus clock, qualified by reset.
    event clock_rise is true(not reset_asserted) @unqualified_clock_rise;
    
    -- This event is the falling edge of the bus clock, qualified by reset.
    event clock_fall is true(not reset_asserted) @unqualified_clock_fall;

     
     // default slaves enumeration: DUT first, eVC last    
    keep for each (ss) in passive_slave_names {	// DUT slaves start from 0
        soft ss == (index + 1).as_a(shr_apb_slave_id_t);
    };
    
    keep active_slave_names.size() != 0 => for each (es) in active_slave_names {
        soft es == get_next_slave_id();
    };
      
    get_next_slave_id():shr_apb_slave_id_t is{
        while next_id.as_a(shr_apb_slave_id_t) in active_slave_names {
            next_id += 1;
        };
        result = next_id.as_a(shr_apb_slave_id_t);
        next_id += 1;	
    };     
}; -- unit shr_apb_env_u




'>

