Title        : Top level env unit for eVC
Project      : APB eVC
File name    : shr_apb_memory_map.e
             :
Created On   : 22/04/2002
             :
Developers   : Patrick Oury, Verisity EURL
             :
Purpose      : 
             :
Description  : 
             :
Assumptions  : none
             :
Limitations  : none
             :
Known Errors : none
             :
Notes        : 

----------------------------------------------------------------------------
Revision History :
----------------------------------------------------------------------------
<'
import shr_apb_env;

// ******************************
// shr_apb_map_address_s
// ******************************
struct shr_apb_map_address_s {
	id : shr_apb_slave_id_t;
	start_address	: shr_apb_address_t;
	end_address		: shr_apb_address_t;
};

// ******************************
// shr_apb_memory_segment_s
// ******************************
struct shr_apb_memory_segment_s {
    num:	uint;
    slave_id:   shr_apb_slave_id_t;		// slave ID
    size:     	uint(bits:33);
    start_ad: 	shr_apb_address_t;
    end_ad:   	shr_apb_address_t;
	keep end_ad == (start_ad + size - 1);		
	keep gen (size) before (start_ad, end_ad);
    
	//** Methods
    // belong() -- return TRUE if 'ad' belongs to the segment
    belong(ad: shr_apb_address_t): bool is {
        result = (ad >= start_ad and ad <= end_ad);    
    };

};// memory_segment

struct shr_apb_reserved_space_s {
    name: shr_apb_bus_name_t;
    !map_addrs: list of shr_apb_map_address_s;
};

// ******************************
// apb_memory_map
// ******************************
unit shr_apb_memory_map_u {
    name:		shr_apb_bus_name_t ;
	next_id: int (bits: 16);
        keep soft next_id == 1; 
    segments: 		list of shr_apb_memory_segment_s;
	first_address:      shr_apb_address_t;
        keep soft first_address == 0;
    last_address:       shr_apb_address_t;
    	keep soft last_address == -1;
    slaves_id:		list of shr_apb_slave_id_t;
	keep gen (slaves_id) before (segments);
    default_seg_size: uint(bits:33);
	keep default_seg_size == calc_default_size(first_address,last_address,slaves_id);
    keep soft segments.size() == slaves_id.size() + reserved_space.map_addrs.size() 
										- reserved_space.map_addrs.unique(.id).size();//number of segments = 
																		  //	user segments + segments for eVC slaves
    keep for each  in slaves_id { 
      soft it == (index+1).as_a(shr_apb_slave_id_t);
    };	
    reserved_space: shr_apb_reserved_space_s;
		keep reserved_space == get_enclosing_unit(shr_apb_env_u).reserved_space;
    reserved_space_copy: list of shr_apb_map_address_s;
      keep gen(reserved_space) before (reserved_space_copy);
      keep gen(reserved_space_copy) before (segments);
      keep reserved_space_copy.size() == reserved_space.map_addrs.size();
    	keep for each in reserved_space_copy {
			it == (reserved_space.map_addrs[index]);
		};

    //The memory is carefull to the user constraints set using macros
    keep for each (seg) in segments {
		seg.num == index;
		soft seg.slave_id == slaves_id[index];	     
		seg.slave_id == get_id(index);
		index > reserved_space_copy.size()-1  and reserved_space_copy.size()> 0 =>
			for each (rs) in reserved_space_copy {
			   ((seg.start_ad < rs.start_address and seg.end_ad < rs.start_address)
				or (seg.start_ad > rs.end_address and seg.end_ad > rs.end_address));
			};
		index <= reserved_space_copy.size()-1 and reserved_space_copy.size()>0  =>  
			seg.start_ad == value(reserved_space_copy[index]).start_address 
									and seg.end_ad == value(reserved_space_copy[index]).end_address;
		soft seg.size == default_seg_size; 
		index > reserved_space_copy.size() => seg.start_ad > prev.end_ad;
                index == 0 => soft seg.start_ad == 0;
                index >  0 => soft seg.start_ad == prev.end_ad + 1;
    };

	get_id(ind : int):shr_apb_slave_id_t is {
		if reserved_space_copy.size() <= ind {
			gen result keeping {
				it == get_unique_evc_slave_id();  
			};
		} else {
			return reserved_space_copy[ind].id;
		};
	};
	
	get_unique_evc_slave_id(): shr_apb_slave_id_t is {
		while next_id.as_a(shr_apb_slave_id_t) in reserved_space_copy.id {
			next_id += 1;
		};
		result = next_id.as_a(shr_apb_slave_id_t);
		next_id += 1;	
	};


    // --------------------------------------------------------------
    // calc_default_size()
    // calculate the default size of a segment in an 
    // evenly destributed memory map,
    // the size is a multiplicity of 1k
    // --------------------------------------------------------------
    calc_default_size(first_addr: shr_apb_address_t,last_addr: shr_apb_address_t,
        slaves: list of shr_apb_slave_id_t): uint(bits:33) is {
        var raw_size: uint(bits:33);
		raw_size =0;
        for each (rs) in reserved_space_copy {
			raw_size += (rs.end_address - rs.start_address);
		};
        raw_size = (  last_address.as_a(uint(bits: 33))
		    - first_address.as_a(uint(bits: 33)) + 1 - raw_size) 
		   / (2 *slaves.size());
        result = raw_size - ( raw_size % 1k);
    };
	

    create_undef_seg(start_addr: shr_apb_address_t,end_addr: shr_apb_address_t) is {
        var seg: shr_apb_memory_segment_s = new shr_apb_memory_segment_s with {
            .num       = segments.size();
            .slave_id  = UNDEFINED;  
            .size      = end_addr - start_addr + 1;
            .start_ad  = start_addr;
            .end_ad    = end_addr;
        };
        segments.add(seg);
    };	 

   post_generate() is also {
        var sorted_mem : list of shr_apb_memory_segment_s = segments.sort(.start_ad);
        for each (seg) in sorted_mem {
            if index != 0 {
                if (sorted_mem[index - 1].end_ad + 1) < seg.start_ad {
                    create_undef_seg(sorted_mem[index - 1].end_ad + 1,
				     seg.start_ad - 1);
                };
            } else {
                if seg.start_ad != 0x00000000 {
                    create_undef_seg(0x00000000,seg.start_ad - 1);
                };
            };
        };
        
        if not segments.is_empty() {
            var end_addr: shr_apb_address_t = segments.sort(.start_ad)
					.last(TRUE).end_ad;
            if  end_addr != 0xffffffff {
                create_undef_seg(end_addr + 1,0xffffffff);
            };
        };
        //active_slaves_id = segments.slave_id.sort(it).unique(it);
    };
  

    post_generate() is also {
        var sorted_mem : list of shr_apb_memory_segment_s = segments.sort(.start_ad);
        for each (seg) in sorted_mem {
	    if index > 0 and seg.start_ad <= sorted_mem[index-1].end_ad {
                error(
                    appendf("%s%s%s\n%s%s%s%s\n %s%s0x%-8.8x%s0x%-8.8x\n %s%s0x%-8.8x%s0x%-8.8x%s%s%s%s%s%s%s\n",
			    "\n*********************************************",
			    "**********",
			    "\nCONFIGURATION ERROR:",
			    "\ncollision between slave ",
			    sorted_mem[index - 1].slave_id,
			    " and slave ",seg.slave_id,
			    sorted_mem[index - 1].slave_id,
			    " start addr: ",sorted_mem[index - 1].start_ad,
			    " end addr: ",sorted_mem[index - 1].end_ad,
			    seg.slave_id,
			    " start addr: ",seg.start_ad,
			    " end addr: ",seg.end_ad,
			    "\n\n address space ",seg.start_ad," to ",
			    sorted_mem[index - 1].end_ad," is overlapping.",
			    "\n*********************************************",
			    "**********"));
            };
        };
    };
	
   post_generate() is also {
        for each (slave) in slaves_id {
            if not segments.has(.slave_id == slave) {
                error(
                    appendf("%s%s%s%s%s%s%s%s",
			    "\n*********************************************",
			    "**********",
			    "\nCONFIGURATION ERROR:",
			    "\n\nslave id ",slave,
			    " was not configured any memory space",
			    "\n*********************************************",
			    "**********"));
            };
        };
    };
    
    
    
    
    
    // ------------------------------------------------------------
    // get_segment(ad: apb_address): apb_memory_segment
    // return the segment in 'segments' that contain 'ad'
    // ------------------------------------------------------------
    get_segment(ad: shr_apb_address_t): shr_apb_memory_segment_s is {
        result = segments.first(.belong(ad));
    };

    // ------------------------------------------------------------
    // get_slave_id()
    // returns the correct slave id according to the address
    // ------------------------------------------------------------
	
	get_id_from_idx(num : uint): shr_apb_slave_id_t is {
		if (num == 0){ return  UNDEFINED};
		result = segments[num].slave_id
	};
	
    get_slave_id(ad : shr_apb_address_t): shr_apb_slave_id_t is {
        result = segments.first(.belong(ad)).slave_id;
    };
	
	get_slave_number(ad : shr_apb_address_t): uint(bits:4) is {
        result = get_slave_id(ad).as_a(int) - 1; 
    };
};

define <apb_set_slave_addr_with_size'statement> "apb_set <env'name> <slave'name> slave address space: start at <start'num> size <size'num>" as {
    extend <env'name> shr_apb_reserved_space_s {
		post_generate() is also {
			var newmap : shr_apb_map_address_s = new shr_apb_map_address_s;
			newmap.start_address = <start'num>;
			newmap.end_address = <start'num> + <size'num> - 1;
			newmap.id = <slave'name>;
			map_addrs.add(newmap);
			map_addrs = map_addrs.sort(.start_address);
		};
    };
};

define <apb_set_slave_addr_with_end'statement> "apb_set <env'name> <slave'name> slave address space: start at <start'num> end at <end'num>" as {
    extend <env'name> shr_apb_reserved_space_s {
		post_generate() is also {			
			var newmap : shr_apb_map_address_s = new shr_apb_map_address_s;
			newmap.start_address = <start'num>;
			newmap.end_address = <end'num>;
			newmap.id = <slave'name>;
			map_addrs.add(newmap);
			map_addrs = map_addrs.sort(.start_address);
		};
    };
};

'>
