Title        : Top level env unit for eVC
Project      : APB eVC
File name    : shr_apb_monitor.e
             :
Created On   : 22/04/2002
             :
Developers   : Patrick Oury, Verisity EURL
             :
Purpose      : 
             :
Description  : 
             :
Assumptions  : none
             :
Limitations  : none
             :
Known Errors : none
             :
Notes        : 

----------------------------------------------------------------------------
Revision History :
----------------------------------------------------------------------------

<'

-- Circular import
import shr_apb_agent;
import shr_apb_env;

-- Declare the monitor unit for the whole bus
unit shr_apb_bus_monitor_u {

    -- backpointer to the env this monitor is monitoring.
    env : shr_apb_env_u;
    
    !phase : shr_apb_phase_t;
    !num_of_transfers:	uint;
    !cur_slave : int;
    has_protocol_checker : bool;
      keep soft has_protocol_checker;
    smp:		shr_apb_signal_map_u;
        keep smp == env.smp;
    ssmp:		list of shr_apb_slave_signal_map_u;
        keep ssmp == env.ssmp;
	 amba3 : bool;
    // Transfer properties
    !cur_transfer : shr_apb_trans_s;
    !transfers    : list of shr_apb_trans_s;
    !tr_number    : int;     // current transfer number
    cur_transfer_slave : shr_apb_slave_id_t;          // slave target of  transfer
  	 log_transfer() is empty; // For Scoreboard :  This method is empty and is called on tr_ended

    // Events  
    event pclk_f is @env.clock_fall;
    event pclk_r is @env.clock_rise;
    
    event tr_started;
	 event tr_enable;
    event tr_ended;
    event data_changed is change(smp.sig_prdata$)@pclk_f;  // PWDATA changed
    event setting_up is  rise(ssmp[cur_slave].sig_psel$)@pclk_f;
    event enable_up is rise(smp.sig_penable$) and true (ssmp[cur_slave].sig_psel$ ==1)@pclk_f ;
    
	 
	 
	 event new_setup is fall(smp.sig_penable$) and not fall (ssmp[cur_slave].sig_psel$)@pclk_f;
    event go_to_sleep is fall(smp.sig_penable$) and fall (ssmp[cur_slave].sig_psel$)@pclk_f;

    evaluate_state()@pclk_f is {
        state machine phase {
              IDLE => SETUP { wait @setting_up;message(MEDIUM,"1 - IDLE to SETUP "); emit tr_started; };
              SETUP => ENABLE { wait @enable_up;message(MEDIUM,"2 - SETUP to ACCESS/ENABLE ");emit tr_enable; }; 
              ENABLE => IDLE {wait @go_to_sleep;message(MEDIUM,"4 - ACCESS/ENABLE to IDLE ");emit tr_ended;};
              ENABLE => SETUP {wait  @new_setup;message(MEDIUM,"3 - ACCESS/ENABLE to SETUP ");emit tr_ended; emit tr_started;};
        };
    };

    run() is also {
        start evaluate_state();
    };
    
    // big_brother
    on pclk_f {
        for i from 0 to env.slaves.size() - 1{
            if env.ssmp[i].sig_psel$==1 {
                set_cur_slave(i);
                break;
            };
        };
    };

    // Methods
    set_cur_slave(i : int) is {
        cur_slave = i;
    };
        
    // selected_slave()
    selected_slave(): shr_apb_slave_id_t is {
        result = UNDEFINED;
        for i from 0 to env.slaves.size() - 1{
            if env.ssmp[i].sig_psel$ == 1 {
                result = env.all_slave_names[i];
                break;
            };
        };
    };
    
    on tr_started {
        num_of_transfers += 1;
        if selected_slave() == UNDEFINED {
            dut_error("ERR_APB_ADDRESS_SLAVE: bridge is sending to an undefined slave");
        };
    };

    on tr_started {  
      cur_transfer = new shr_apb_trans_s with {
			.addr = smp.sig_paddr$;
			.direction = smp.sig_pwrite$;
			.data = smp.sig_pwdata$;      
		   .slave_number = selected_slave();
			
		};
      cur_transfer.start();
		  message(MEDIUM,"tr started");
    };// tr_started

    on tr_ended { 
        // update transfer data
        if cur_transfer != NULL {
            if cur_transfer.direction == READ {
                cur_transfer.data = smp.sig_prdata$; // write data
					 if bind(smp.sig_pslverr, external) {
					 	 cur_transfer.err = smp.sig_pslverr$.as_a(bool);
					 };
            };
         	message(MEDIUM,"finished transfer to ",cur_transfer.slave_number, " ",cur_transfer.copy());
         	if env.log_enable {
          		transfers.add(cur_transfer.copy());
        		};
        };
		  log_transfer(); // The transfer is updated so the user can put it in the scoreboard        
    };// on tr_ended    	
    
    short_name(): string is {
        result = append("Monitor");
    };
	 short_name_style(): vt_style is {
        result = DARK_GREEN;
    };	    
    
};

extend sys {
    setup() is also {
        specman("set check \"/ERR_APB_ADDRESS_SLAVE/\" WARNING");
    };	

}; -- unit shr_apb_bus_monitor_u
    
'>

